@@ -1,23 +1,24 @@
 //! Configuration used by PyO3 for conditional support of varying Python versions.
 //!
-//! This crate exposes two functions, [`use_pyo3_cfgs`] and [`add_extension_module_link_args`],
-//! which are intended to be called from build scripts to simplify building crates which depend on
-//! PyO3.
+//! This crate exposes functionality to be called from build scripts to simplify building crates
+//! which depend on PyO3.
 //!
 //! It used internally by the PyO3 crate's build script to apply the same configuration.
 
 mod errors;
 mod impl_;
 
+#[cfg(feature = "resolve-config")]
 use std::io::Cursor;
+use std::{env, process::Command};
 
+#[cfg(feature = "resolve-config")]
 use once_cell::sync::OnceCell;
 
-use impl_::InterpreterConfig;
-
-// Used in `pyo3-macros-backend`; may expose this in a future release.
-#[doc(hidden)]
-pub use impl_::PythonVersion;
+pub use impl_::{
+    cross_compiling, find_all_sysconfigdata, parse_sysconfigdata, BuildFlag, BuildFlags,
+    CrossCompileConfig, InterpreterConfig, PythonImplementation, PythonVersion,
+};
 
 /// Adds all the [`#[cfg]` flags](index.html) to the current compilation.
 ///
@@ -27,11 +28,12 @@ pub use impl_::PythonVersion;
 ///
 /// | Flag | Description |
 /// | ---- | ----------- |
-/// | `#[cfg(Py_3_6)]`, `#[cfg(Py_3_7)]`, `#[cfg(Py_3_8)]`, `#[cfg(Py_3_9)]`, `#[cfg(Py_3_10)]` | These attributes mark code only for a given Python version and up. For example, `#[cfg(Py_3_6)]` marks code which can run on Python 3.6 **and newer**. |
+/// | `#[cfg(Py_3_7)]`, `#[cfg(Py_3_8)]`, `#[cfg(Py_3_9)]`, `#[cfg(Py_3_10)]` | These attributes mark code only for a given Python version and up. For example, `#[cfg(Py_3_7)]` marks code which can run on Python 3.7 **and newer**. |
 /// | `#[cfg(Py_LIMITED_API)]` | This marks code which is run when compiling with PyO3's `abi3` feature enabled. |
 /// | `#[cfg(PyPy)]` | This marks code which is run when compiling for PyPy. |
 ///
 /// For examples of how to use these attributes, [see PyO3's guide](https://pyo3.rs/latest/building_and_distribution/multiple_python_versions.html).
+#[cfg(feature = "resolve-config")]
 pub fn use_pyo3_cfgs() {
     get().emit_pyo3_cfgs();
 }
@@ -60,6 +62,7 @@ fn _add_extension_module_link_args(target_os: &str, mut writer: impl std::io::Wr
 ///
 /// Because this will never change in a given compilation run, this is cached in a `once_cell`.
 #[doc(hidden)]
+#[cfg(feature = "resolve-config")]
 pub fn get() -> &'static InterpreterConfig {
     static CONFIG: OnceCell<InterpreterConfig> = OnceCell::new();
     CONFIG.get_or_init(|| {
@@ -78,23 +81,28 @@ pub fn get() -> &'static InterpreterConfig {
 
 /// Path where PyO3's build.rs will write configuration by default.
 #[doc(hidden)]
+#[cfg(feature = "resolve-config")]
 const DEFAULT_CROSS_COMPILE_CONFIG_PATH: &str =
     concat!(env!("OUT_DIR"), "/pyo3-cross-compile-config.txt");
 
 /// Build configuration provided by `PYO3_CONFIG_FILE`. May be empty if env var not set.
 #[doc(hidden)]
+#[cfg(feature = "resolve-config")]
 const CONFIG_FILE: &str = include_str!(concat!(env!("OUT_DIR"), "/pyo3-build-config-file.txt"));
 
 /// Build configuration set if abi3 features enabled and `PYO3_NO_PYTHON` env var present. Empty if
 /// not both present.
 #[doc(hidden)]
+#[cfg(feature = "resolve-config")]
 const ABI3_CONFIG: &str = include_str!(concat!(env!("OUT_DIR"), "/pyo3-build-config-abi3.txt"));
 
 /// Build configuration discovered by `pyo3-build-config` build script. Not aware of
 /// cross-compilation settings.
 #[doc(hidden)]
+#[cfg(feature = "resolve-config")]
 const HOST_CONFIG: &str = include_str!(concat!(env!("OUT_DIR"), "/pyo3-build-config.txt"));
 
+#[cfg(feature = "resolve-config")]
 fn abi3_config() -> InterpreterConfig {
     let mut interpreter_config = InterpreterConfig::from_reader(Cursor::new(ABI3_CONFIG))
         .expect("failed to parse hardcoded PyO3 abi3 config");
@@ -108,14 +116,47 @@ fn abi3_config() -> InterpreterConfig {
     interpreter_config
 }
 
+/// Use certain features if we detect the compiler being used supports them.
+///
+/// Features may be removed or added as MSRV gets bumped or new features become available,
+/// so this function is unstable.
+#[doc(hidden)]
+pub fn print_feature_cfgs() {
+    fn rustc_minor_version() -> Option<u32> {
+        let rustc = env::var_os("RUSTC")?;
+        let output = Command::new(rustc).arg("--version").output().ok()?;
+        let version = core::str::from_utf8(&output.stdout).ok()?;
+        let mut pieces = version.split('.');
+        if pieces.next() != Some("rustc 1") {
+            return None;
+        }
+        pieces.next()?.parse().ok()
+    }
+
+    let rustc_minor_version = rustc_minor_version().unwrap_or(0);
+
+    // Enable use of const generics on Rust 1.51 and greater
+    if rustc_minor_version >= 51 {
+        println!("cargo:rustc-cfg=min_const_generics");
+    }
+
+    // Enable use of std::ptr::addr_of! on Rust 1.51 and greater
+    if rustc_minor_version >= 51 {
+        println!("cargo:rustc-cfg=addr_of");
+    }
+}
+
 /// Private exports used in PyO3's build.rs
 ///
 /// Please don't use these - they could change at any time.
 #[doc(hidden)]
 pub mod pyo3_build_script_impl {
+    #[cfg(feature = "resolve-config")]
     use crate::errors::{Context, Result};
+    #[cfg(feature = "resolve-config")]
     use std::path::Path;
 
+    #[cfg(feature = "resolve-config")]
     use super::*;
 
     pub mod errors {
@@ -130,6 +171,7 @@ pub mod pyo3_build_script_impl {
     /// Differs from .get() above only in the cross-compile case, where PyO3's build script is
     /// required to generate a new config (as it's the first build script which has access to the
     /// correct value for CARGO_CFG_TARGET_OS).
+    #[cfg(feature = "resolve-config")]
     pub fn resolve_interpreter_config() -> Result<InterpreterConfig> {
         if !CONFIG_FILE.is_empty() {
             InterpreterConfig::from_reader(Cursor::new(CONFIG_FILE))
